@@ -1,12 +1,17 @@
 # coding=utf-8
 """Ground Heat Exchanger (GHE) in a district thermal system."""
-from .._base import _GeometryBase
+import math
+import json
 
-from ladybug_geometry.geometry2d.polygon import Polygon2D
+from ladybug_geometry.geometry2d import Vector2D, Point2D, Polygon2D
+from ladybug_geometry.geometry3d import Point3D, Vector3D, Plane, Face3D
 from honeybee.typing import valid_string, float_positive, float_in_range, int_in_range
+from honeybee.units import conversion_factor_to_meters
 from honeybee.altnumber import autocalculate
 from dragonfly.projection import polygon_to_lon_lat
 
+from .._base import _GeometryBase
+
 
 class GroundHeatExchanger(_GeometryBase):
     """Represents a Ground Heat Exchanger in a district thermal system.
@@ -15,20 +20,61 @@ class GroundHeatExchanger(_GeometryBase):
         identifier: Text string for a unique heat exchanger ID. Must contain only
             characters that are acceptable in OpenDSS. This will be used to
             identify the object across the exported geoJSON and OpenDSS files.
-        geometry: A Polygon2D representing the geometry of the heat exchanger.
+        geometry: A Polygon2D or horizontal Face3D representing the geometry
+            of the heat exchanger.
 
     Properties:
         * identifier
         * display_name
         * geometry
+        * boundary_2d
+        * holes_2d
     """
+    # a list of GHE properties that are needed to represent the GHE in EnergyPlus
+    PROPERTY_NAMES = (
+        'Borehole Length (m)',
+        'Borehole Radius (m)',
+        'Design Flow Rate (m3/s)',
+        'Ground Temperature (C)',
+        'Ground Conductivity (W/m-K)',
+        'Ground Heat Capacity (J/m3-K)',
+        'Grout Conductivity (W/m-K)',
+        'Number of Boreholes',
+        'Pipe Outer Diameter (m)',
+        'Pipe Conductivity (W/m-K)',
+        'Pipe Thickness (m)',
+        'U Tube Distance (m)'
+    )
+    # a list of corresponding fields in the GHEDesigner summary JSON
+    PROPERTY_PATHS = (
+        ('ghe_system', 'active_borehole_length', 'value'),
+        ('ghe_system', 'borehole_diameter', 'value'),
+        ('ghe_system', 'fluid_mass_flow_rate_per_borehole', 'value'),
+        ('ghe_system', 'soil_undisturbed_ground_temp', 'value'),
+        ('ghe_system', 'soil_thermal_conductivity', 'value'),
+        ('ghe_system', 'soil_volumetric_heat_capacity', 'value'),
+        ('ghe_system', 'grout_thermal_conductivity', 'value'),
+        ('ghe_system', 'total_drilling', 'value'),
+        ('ghe_system', 'pipe_geometry', 'pipe_outer_diameter', 'value'),
+        ('ghe_system', 'pipe_thermal_conductivity', 'value'),
+        ('ghe_system', 'pipe_geometry', 'pipe_inner_diameter', 'value'),
+        ('ghe_system', 'shank_spacing', 'value')
+    )
+
     __slots__ = ()
 
     def __init__(self, identifier, geometry):
         """Initialize GroundHeatExchanger."""
         _GeometryBase.__init__(self, identifier)  # process the identifier
-        assert isinstance(geometry, Polygon2D), 'Expected ladybug_geometry ' \
-            'Polygon2D for GroundHeatExchanger geometry. Got {}'.format(type(geometry))
+        assert isinstance(geometry, (Polygon2D, Face3D)), 'Expected ladybug_geometry ' \
+            'Polygon2D or Face3D for GroundHeatExchanger geometry. ' \
+            'Got {}'.format(type(geometry))
+        if isinstance(geometry, Face3D):
+            if geometry.normal.z < 0:  # ensure upward-facing Face3D
+                geometry = geometry.flip()
+            # ensure a global 2D origin, which helps in solve adjacency and the dict schema
+            o_pl = Plane(Vector3D(0, 0, 1), Point3D(0, 0, geometry.plane.o.z))
+            geometry = Face3D(geometry.boundary, o_pl, geometry.holes)
         self._geometry = geometry
 
     @classmethod
@@ -41,7 +87,9 @@ class GroundHeatExchanger(_GeometryBase):
         # check the type of dictionary
         assert data['type'] == 'GroundHeatExchanger', 'Expected GroundHeatExchanger ' \
             'dictionary. Got {}.'.format(data['type'])
-        geo = Polygon2D.from_dict(data['geometry'])
+        geo = Face3D.from_dict(data['geometry']) \
+            if data['geometry']['type'] == 'Face3D' \
+            else Polygon2D.from_dict(data['geometry'])
         trans = cls(data['identifier'], geo)
         if 'display_name' in data and data['display_name'] is not None:
             trans.display_name = data['display_name']
@@ -62,15 +110,30 @@ class GroundHeatExchanger(_GeometryBase):
             conversion_factors: A tuple with two values used to translate between
                 meters and longitude, latitude.
         """
-        geo = cls._geojson_coordinates_to_polygon2d(
+        geo = cls._geojson_coordinates_to_face3d(
             data['geometry']['coordinates'], origin_lon_lat, conversion_factors)
         return cls(data['properties']['id'], geo)
 
     @property
     def geometry(self):
-        """Get a Polygon2D representing the ground heat exchanger."""
+        """Get a Polygon2D or Face3D representing the ground heat exchanger."""
         return self._geometry
 
+    @property
+    def boundary_2d(self):
+        """Get a Polygon2D for the outer boundary of the ground heat exchanger."""
+        return self._geometry.boundary_polygon2d \
+            if isinstance(self._geometry, Face3D) else self._geometry
+
+    @property
+    def hole_polygon2d(self):
+        """Get a list of Polygon2D for the holes in the ground heat exchanger.
+
+        Will be None if the ground heat exchanger has no holes in it.
+        """
+        return self._geometry.hole_polygon2d \
+            if isinstance(self._geometry, Face3D) else None
+
     def to_dict(self):
         """GroundHeatExchanger dictionary representation."""
         base = {'type': 'GroundHeatExchanger'}
@@ -92,9 +155,18 @@ class GroundHeatExchanger(_GeometryBase):
             conversion_factors: A tuple with two values used to translate between
                 meters and longitude, latitude.
         """
-        pts = [(pt.x, pt.y) for pt in self.geometry.vertices]
-        coords = [polygon_to_lon_lat(pts, origin_lon_lat, conversion_factors)]
-        coords[0].append(coords[0][0])
+        if isinstance(self.geometry, Face3D):
+            loops = [[(pt.x, pt.y) for pt in self.geometry.boundary]]
+            if self.geometry.has_holes:
+                for hole in self.geometry.holes:
+                    loops.append([(pt.x, pt.y) for pt in hole])
+        else:
+            loops = [[(pt.x, pt.y) for pt in self.geometry.vertices]]
+        coords = []
+        for pts in loops:
+            loop = polygon_to_lon_lat(pts, origin_lon_lat, conversion_factors)
+            loop.append(loop[0])
+            coords.append(loop)
         return {
             'type': 'Feature',
             'properties': {
@@ -113,6 +185,145 @@ class GroundHeatExchanger(_GeometryBase):
             }
         }
 
+    def load_boreholes(self, borehole_file, units='Meters', ortho_rotation=False):
+        """Load borehole positions for this GHE from a borehole file made by GHEDesigner.
+
+        Args:
+            borehole_file: Full path to a BoreFieldData.csv produced by GHEDesigner
+                from this GroundHeatExchanger object.
+            units: The units system in which the geometry of this GroundHeatExchanger
+                object exists. This is used to convert between GHEDesigner's
+                native units of Meters over to the units system of this
+                GroundHeatExchanger object. (Default: Meters).
+            ortho_rotation: A boolean to note whether this GroundHeatExchanger
+                geometry was rotated to have it's right-angles align with the
+                coordinate system of GHEDesigner. This should be False when this
+                GroundHeatExchanger's geometry was directly translated to
+                GHEDesigner and should be True if the geometry was converted
+                to GeoJSON and then translated to a GeoJSON by the ThermalNetwork
+                package. (Default: False).
+
+        Returns:
+            A list of Point2D for the position of each borehole in the ground heat
+            exchanger. These points should all be contained within the geometry
+            of this object.
+        """
+        # load the borehole positions from the file
+        with open(borehole_file, 'r') as bf:
+            borehole_data = bf.readlines()
+
+        # create the Point2D objects and format them for the units system
+        scale_fac = 1.0 / conversion_factor_to_meters(units)
+        bound_poly = self.boundary_2d
+        min_pt = bound_poly.min
+        ghe_boreholes = []
+        for pt in borehole_data[1:]:
+            b_pt = Point2D(*(float(c) for c in pt.split(',')))
+            b_pt = b_pt.scale(scale_fac)
+            ghe_boreholes.append(b_pt)
+        move_vec_2d = Vector2D(min_pt.x, min_pt.y)
+        ghe_boreholes = [pt.move(move_vec_2d) for pt in ghe_boreholes]
+
+        # sense if the geometry has a right angle and, if so, rotate it
+        if ortho_rotation:
+            if bound_poly.is_clockwise:
+                bound_poly = bound_poly.reverse()
+            # get the point of the polygon representing the lower left corner
+            pt_dists = []
+            for i, point in enumerate(bound_poly):
+                pt_dists.append((min_pt.distance_to_point(point), i))
+            sorted_i = [x for _, x in sorted(pt_dists, key=lambda pair: pair[0])]
+            origin_i = sorted_i[0]
+            origin = bound_poly[origin_i]
+            prev_pt = bound_poly[origin_i - 1]
+            next_pt = bound_poly[origin_i + 1] \
+                if origin_i < len(bound_poly) - 1 else bound_poly[0]
+            # check if there is a need to rotate the polygon
+            if not (origin.x == min_pt.x and origin.y == min_pt.y):
+                vec_1, vec_2 = next_pt - origin, prev_pt - origin
+                if 89 < math.degrees(vec_1.angle(vec_2)) < 91:
+                    # rotate all of the boreholes
+                    y_axis = Vector2D(0, 1)
+                    rot_ang = math.degrees(vec_2.angle_counterclockwise(y_axis))
+                    rot_ang = rot_ang - 360 if rot_ang > 180 else rot_ang
+                    ghe_boreholes = [pt.rotate(math.radians(-rot_ang), origin)
+                                     for pt in ghe_boreholes]
+        return ghe_boreholes
+
+    @staticmethod
+    def load_g_function(g_func_file):
+        """Load the G-Function for this GHE from a G function file made by GHEDesigner.
+
+        Args:
+            g_func_file: Full path to a Gfunction.csv produced by GHEDesigner
+                from this GroundHeatExchanger object.
+
+        Returns:
+            A list of lists of G-function coefficients that describe the response
+            of the ground to the input loads. Each pair of factors represents
+            a point on the G-function. Flattening this list of lists yields
+            properties that can be plugged into the EnergyPlus G-function object.
+        """
+        with open(g_func_file, 'r') as gf:
+            g_function = gf.readlines()
+        g_function = [[float(v) for v in line.split(',')[:2]] for line in g_function[1:]]
+        return g_function
+
+    @staticmethod
+    def load_energyplus_properties(summary_file):
+        """Load E+ properties for this GHE from a summary file made by GHEDesigner.
+
+        Args:
+            summary_file: Full path to a SimulationSummary.json produced by GHEDesigner
+                from this GroundHeatExchanger object.
+
+        Returns:
+            A list of properties of the ground heat exchanger that are needed to
+            simulate it in EnergyPlus. These values output here correspond to the
+            PROPERTY_NAMES on this object.
+        """
+        with open(summary_file, 'r') as sf:
+            summary_data = json.load(sf)
+        properties = []
+        zp = zip(GroundHeatExchanger.PROPERTY_NAMES, GroundHeatExchanger.PROPERTY_PATHS)
+        for p_name, p_path in zp:
+            val = summary_data[p_path[0]]
+            for key in p_path[1:]:
+                val = val[key]
+            if '(J/m3-K)' in p_name:
+                val = val * 0.001
+            if 'Borehole Radius' in p_name:
+                val = val * 0.5
+            if 'Number of Boreholes' in p_name:
+                val = int(val / properties[0])
+            properties.append(val)
+        properties[-2] = properties[-4] - properties[-2]  # pipe thickness from diameter
+        # compute total system flow rate from per-borehole rate
+        density = summary_data['ghe_system']['fluid_density']['value']
+        properties[2] = (properties[2] * properties[7]) / density
+        return properties
+
+    @staticmethod
+    def load_monthly_temperatures(summary_file):
+        """Load ground temperatures for this GHE from a summary file made by GHEDesigner.
+
+        Args:
+            summary_file: Full path to a SimulationSummary.json produced by GHEDesigner
+                from this GroundHeatExchanger object.
+
+        Returns:
+            A list of ground temperatures in Celsius with one value for each month
+            of the period over which the GHEDesigner simulation was run (typically
+            20 years). This can be used to check the drift in the ground temperature
+            over long periods of time.
+        """
+        with open(summary_file, 'r') as sf:
+            sum_data = json.load(sf)
+        temperatures = []
+        for month_data in sum_data['simulation_results']['monthly_temp_summary']['data']:
+            temperatures.append(month_data[1])
+        return temperatures
+
     def __copy__(self):
         new_ghe = GroundHeatExchanger(self.identifier, self.geometry)
         new_ghe._display_name = self._display_name
@@ -463,8 +674,8 @@ class PipeParameter(object):
         out_d = data['outer_diameter'] if 'outer_diameter' in data else 0.0266
         s_spc = data['shank_spacing'] if 'shank_spacing' in data else 0.0323
         rough = data['roughness'] if 'roughness' in data else 1e-06
-        cond = data['conductivity'] if 'conductivity' in data else 2.3
-        cap = data['heat_capacity'] if 'heat_capacity' in data else 2343500
+        cond = data['conductivity'] if 'conductivity' in data else 0.4
+        cap = data['heat_capacity'] if 'heat_capacity' in data else 1542000
         arr = data['arrangement'] if 'arrangement' in data else 'SingleUTube'
         return cls(in_d, out_d, s_spc, rough, cond, cap, arr)
 
@@ -787,6 +998,14 @@ class GHEDesignParameter(object):
         month_count: An integer for the number of months over which the simulation
             will be run in order to ensure stable ground temperature
             conditions. (Default: 240).
+        method: Text for the method to be used in sizing the GHE. Downstream
+            will size each GHE for all of the buildings that lie downstream of
+            the GHE. AreaProportional will take the load along the entire loop
+            and evenly distribute it per unit area of GHE. (Default: AreaProportional).
+            Choose from the followingL
+
+            * AreaProportional
+            * Downstream
 
     Properties:
         * flow_rate
@@ -794,18 +1013,22 @@ class GHEDesignParameter(object):
         * max_eft
         * min_eft
         * month_count
+        * method
     """
-    __slots__ = ('_flow_rate', '_flow_type', '_max_eft', '_min_eft', '_month_count')
+    __slots__ = ('_flow_rate', '_flow_type', '_max_eft', '_min_eft',
+                 '_month_count', '_method')
     FLOW_TYPES = ('Borehole', 'System')
+    METHODS = ('AreaProportional', 'Downstream')
 
     def __init__(self, flow_rate=0.2, flow_type='Borehole', max_eft=35, min_eft=5,
-                 month_count=240):
+                 month_count=240, method='AreaProportional'):
         """Initialize BoreholeParameter."""
         self.flow_rate = flow_rate
         self.flow_type = flow_type
         self._min_eft = float_positive(min_eft, 'GHE min entering fluid temperature')
         self.max_eft = max_eft
         self.month_count = month_count
+        self.method = method
 
     @classmethod
     def from_dict(cls, data):
@@ -823,7 +1046,8 @@ class GHEDesignParameter(object):
             'flow_type': 'Borehole',  # text for the type of object flow_rate references
             'max_eft': 35,  # float for max entering fluid temperature in C
             'min_eft': 5,  # float for min entering fluid temperature in C
-            'month_count': 240  # int for the number of months to run the simulation
+            'month_count': 240,  # int for the number of months to run the simulation
+            'method': 'Downstream'  # text for the sizing method
             }
         """
         flow_rate = data['flow_rate'] if 'flow_rate' in data else 0.2
@@ -831,7 +1055,8 @@ class GHEDesignParameter(object):
         max_eft = data['max_eft'] if 'max_eft' in data else 35
         min_eft = data['min_eft'] if 'min_eft' in data else 5
         month_count = data['month_count'] if 'month_count' in data else 240
-        return cls(flow_rate, flow_type, max_eft, min_eft, month_count)
+        method = data['method'] if 'method' in data else 'AreaProportional'
+        return cls(flow_rate, flow_type, max_eft, min_eft, month_count, method)
 
     @property
     def flow_rate(self):
@@ -895,6 +1120,30 @@ class GHEDesignParameter(object):
     def month_count(self, value):
         self._month_count = int_in_range(value, 12, input_name='GHE month count')
 
+    @property
+    def method(self):
+        """Get or set text for the sizing method.
+
+        Choose from the following options:
+
+        * AreaProportional
+        * Downstream
+        """
+        return self._method
+
+    @method.setter
+    def method(self, value):
+        clean_input = valid_string(value).lower()
+        for key in self.METHODS:
+            if key.lower() == clean_input:
+                value = key
+                break
+        else:
+            raise ValueError(
+                'Design method {} is not recognized.\nChoose from the '
+                'following:\n{}'.format(value, self.METHODS))
+        self._method = value
+
     def to_dict(self):
         """Get GHEDesignParameter dictionary."""
         base = {'type': 'GHEDesignParameter'}
@@ -903,6 +1152,7 @@ class GHEDesignParameter(object):
         base['min_eft'] = self.min_eft
         base['max_eft'] = self.max_eft
         base['month_count'] = self.month_count
+        base['method'] = self.method
         return base
 
     def duplicate(self):
@@ -911,7 +1161,8 @@ class GHEDesignParameter(object):
 
     def __copy__(self):
         return GHEDesignParameter(
-            self.flow_rate, self.flow_type, self.min_eft, self.max_eft, self.month_count)
+            self.flow_rate, self.flow_type, self.max_eft, self.min_eft,
+            self.month_count, self.method)
 
     def _eft_check(self):
         """Check that max_eft is greater than or equal to min_eft."""
